#ifndef CGROM_H
#define CGROM_H

/*
Copyright(c) 2008-2009, Jochen Arndt <joe127@t-online.de>

This program is distributed under the terms of the GNU General Public License.
See COPYING for additional information.

To test the display compile fritzmon.c with TEST_CMD defined and 
execute 'fritzmon -t <0-f>'. The display will then show 16 characters starting
at the passed hex parameter multiplied by 16.
E.g. 'fritzmon -t 1' will display the block characters from 0x10 to 0x1F.
*/

// Kathrein UFS-910 CGROM character indices
// Codes 0x00 to 0x07 are CGRAM addresses (CGRAM defined patterns)
//  which are set to 'äöüÄÖÜß' by default.
//				     UTF-8 HTML
#define CGROM_utri	0x08	// ▲ e2 96 b2		BLACK UP-POINTING TRIANGLE
#define CGROM_dtri	0x09	// ▼ e2 96 bc		BLACK DOWN-POINTING TRIANGLE
#define CGROM_ltri	0x0A	// ◀ e2 97 80		BLACK LEFT-POINTING TRIANGLE
#define CGROM_rtri	0x0B	// ▶ e2 96 b6		BLACK RIGHT-POINTING TRIANGLE
#define CGROM_llt	0x0C	// ≪ e2 89 aa		MUCH LESS-THAN (<<)
#define CGROM_ggt	0x0D	// ≫ e2 89 ab		MUCH GREATER-THAN  (>>)
#define CGROM_vdbar	0x0E	//			VERTICAL THICK DOUBLE BAR (||)  
#define CGROM_rect	0x0F	// ▮ e2 96 ae		BLACK VERTICAL RECTANGLE
#define CGROM_vrbar0	0x10	//			VERTICAL BAR RIGHT WIDTH 0 = empty
#define CGROM_vrbar1	0x11	//			VERTICAL BAR RIGHT WIDTH 1
#define CGROM_vrbar2	0x12	//			VERTICAL BAR RIGHT WIDTH 2
#define CGROM_vrbar3	0x13	//			VERTICAL BAR RIGHT WIDTH 3
#define CGROM_vrbar4	0x14	//			VERTICAL BAR RIGHT WIDTH 4
#define CGROM_vrbar5	0x15	// █ e2 96 88		VERTICAL BAR RIGHT WIDTH 5 = FULL BLOCK
#define CGROM_block7	0x15	// █ e2 96 88		BOTTOM BLOCK HEIGHT 7 = FULL BLOCK
#define CGROM_block6	0x16	//			BOTTOM BLOCK HEIGHT 6
#define CGROM_block5	0x17	//			BOTTOM BLOCK HEIGHT 5
#define CGROM_block4	0x18	//			BOTTOM BLOCK HEIGHT 4
#define CGROM_block3	0x19	//			BOTTOM BLOCK HEIGHT 3
#define CGROM_block2	0x1A	//			BOTTOM BLOCK HEIGHT 2
#define CGROM_block1	0x1B	// _			BOTTOM BLOCK HEIGHT 1 = UNDERSCORE
#define CGROM_block0	0x1C	//			BOTTOM BLOCK HEIGHT 0 = empty
#define CGROM_shade1	0x1D	// 			SHADED BLOCK
#define CGROM_shade2	0x1E	// 			SHADED BLOCK
#define CGROM_diamond	0x1F	//   e2 97 86		BLACK DIAMOND
// Codes 0x20 to 0x7E are US-ASCII
#define CGROM_block	0x7F	// █ e2 96 88		FULL BLOCK
#define	CGROM_Ccedil	0x80	// Ç c3 87 &Ccedil;	LATIN CAPITAL LETTER C WITH CEDILLA
#define	CGROM_uuml	0x81	// ü c3 bc &uuml;	LATIN SMALL LETTER U WITH DIAERESIS
#define CGROM_eacute	0x82	// é c3 a9 &eacute;	LATIN SMALL LETTER E WITH ACUTE
#define CGROM_atilde	0x83	// ã c3 a3 &atilde; 	LATIN SMALL LETTER A WITH TILDE
#define CGROM_amacr	0x83	// ā c4 81 		LATIN SMALL LETTER A WITH MACRON
#define CGROM_auml	0x84	// ä c3 a4 &auml;	LATIN SMALL LETTER A WITH DIAERESIS
#define CGROM_agrave	0x85	// à c3 a0 &agrave;	LATIN SMALL LETTER A WITH GRAVE
#define CGROM_aring	0x86	// å c3 a5 &aring;	LATIN SMALL LETTER A WITH RING ABOVE
#define CGROM_ccedil	0x87	// ç c3 a7 &ccedil;	LATIN SMALL LETTER C WITH CEDILLA
#define CGROM_sigmaf	0x87	// ς cf 82 &sigmaf;	GREEK SMALL LETTER FINAL SIGMA
#define CGROM_emacr	0x88	// ē c4 93		LATIN SMALL LETTER E WITH MACRON
#define CGROM_euml	0x89	// ë c3 ab &euml;	LATIN SMALL LETTER E WITH DIAERESIS
#define CGROM_egrave	0x8A	// è c3 a8 &egrave;	LATIN SMALL LETTER E WITH GRAVE
#define CGROM_iuml	0x8B	// ï c3 af &iuml;	LATIN SMALL LETTER I WITH DIAERESIS
#define CGROM_imacr	0x8C	// ī c4 ab		LATIN SMALL LETTER I WITH MACRON
#define CGROM_igrave	0x8D	// ì c3 ac &igrave;	LATIN SMALL LETTER I WITH GRAVE
#define CGROM_Auml	0x8E	// Ä c3 84 &Auml;	LATIN CAPITAL LETTER A WITH DIAERESIS
#define CGROM_Acirc	0x8F	// Â c3 82 &Acirc;	LATIN CAPITAL LETTER A WITH CIRCUMFLEX
#define CGROM_Aring	0x8F	// Å c3 85 &Aring;	LATIN CAPITAL LETTER A WITH RING ABOVE
#define CGROM_Eacute	0x90	// É c3 89 &Eacute;	LATIN CAPITAL LETTER E WITH ACUTE
#define CGROM_aelig	0x91	// æ c3 a6 &aelig;	LATIN SMALL LETTER AE
#define CGROM_Aelig	0x92	// Æ c3 86 &AElig;	LATIN CAPITAL LETTER AE
#define CGROM_otilde	0x93	// õ c3 b5 &otilde;	LATIN SMALL LETTER O WITH TILDE
#define CGROM_omacr	0x93	// ō c5 8d		LATIN SMALL LETTER O WITH MACRON
#define CGROM_ouml	0x94	// ö c3 b6 &ouml;	LATIN SMALL LETTER O WITH DIAERESIS
#define CGROM_ograve	0x95	// ò c3 b2 &ograve;	LATIN SMALL LETTER O WITH GRAVE
#define CGROM_umacr	0x96	// ū c5 ab		LATIN SMALL LETTER U WITH MACRON
#define CGROM_ugrave	0x97	// ù c3 b9 &ugrave;	LATIN SMALL LETTER U WITH GRAVE
#define CGROM_yuml	0x98	// ÿ c3 bf &yuml;	LATIN SMALL LETTER Y WITH DIAERESIS
#define CGROM_Ouml	0x99	// Ö c3 96 &Ouml;	LATIN CAPITAL LETTER O WITH DIAERESIS
#define CGROM_Uuml	0x9A	// Ü c3 9c &Uuml;	LATIN CAPITAL LETTER U WITH DIAERESIS
#define CGROM_cent	0x9B	// ¢ c2 a2 &cent;	CENT SIGN
#define CGROM_pound	0x9C	// £ c2 a3 &pound;	POUND SIGN
#define CGROM_yen	0x9D	// ¥ c2 a5 &yen;	YEN SIGN
#define CGROM_pt	0x9E	// ₧ e2 82 a7		PESETA SIGN
#define CGROM_fnof	0x9F	// ƒ c6 92 &fnof;	LATIN SMALL LETTER F WITH HOOK
#define CGROM_aacute	0xA0	// á c3 a1 &aacute;	LATIN SMALL LETTER A WITH ACUTE
#define CGROM_iacute	0xA1	// í c3 ad &iacute;	LATIN SMALL LETTER I WITH ACUTE
#define CGROM_oacute	0xA2	// ó c3 b3 &oacute;	LATIN SMALL LETTER O WITH ACUTE
#define CGROM_sigma	0xA2	// σ cf 83 &sigma;	GREEK SMALL LETTER SIGMA
#define CGROM_uacute	0xA3	// ú c3 ba &uacute;	LATIN SMALL LETTER U WITH ACUTE
#define CGROM_ntilde	0xA4	// ñ c3 b1 &ntilde;	LATIN SMALL LETTER N WITH TILDE
#define CGROM_Ntilde	0xA5	// Ñ c3 91 &Ntilde;	LATIN CAPITAL LETTER N WITH TILDE
#define CGROM_ordf	0xA6	// ª c2 aa &ordf;	FEMININE ORDINAL INDICATOR (underlined on UFS)
#define CGROM_ordm	0xA7	// º c2 ba &ordm;	MASCULINE ORDINAL INDICATOR (underlined on UFS)
#define CGROM_iquest	0xA8	// ¿ c2 bf &iquest;	INVERTED QUESTION MARK
#define CGROM_mnot	0xA9	//			mirrored not sign
#define CGROM_not	0xAA	// ¬ c2 ac &not;	NOT SIGN
#define CGROM_frac12	0xAB	// ½ c2 bd &frac12;	VULGAR FRACTION ONE HALF
#define CGROM_frac14	0xAC	// ¼ c2 bc &frac14;	VULGAR FRACTION ONE QUARTER
#define CGROM_iexcl	0xAD	// ¡ c2 a1 &iexcl;	INVERTED EXCLAMATION MARK
#define CGROM_laquo	0xAE	// « c2 ab &laquo;	LEFT-POINTING DOUBLE ANGLE QUOTATION MARK
#define CGROM_raquo	0xAF	// » c2 bb &raquo;	RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK
#define CGROM_alpha	0xB0	// α ce b1 &alpha;	GREEK SMALL LETTER ALPHA
#define CGROM_beta	0xB1	// β ce b2 &beta;	GREEK SMALL LETTER BETA
#define CGROM_szlig	0xB1	// ß c3 9f &szlig;	LATIN SMALL LETTER SHARP S
#define CGROM_gamma	0xB2	// γ ce b3 &gamma;	GREEK SMALL LETTER GAMMA
#define CGROM_Delta	0xB3	// Δ ce 94 &Delta;	GREEK CAPITAL LETTER DELTA
#define CGROM_epsilon	0xB4	// ε ce b5 &epsilon;	GREEK SMALL LETTER EPSILON
#define CGROM_eta	0xB5	// η ce b7 &eta;	GREEK SMALL LETTER ETA
#define CGROM_Theta	0xB6	// Θ ce 98 &Theta;	GREEK CAPITAL LETTER THETA
#define CGROM_lambda	0xB7	// λ ce bb &lambda;	GREEK SMALL LETTER LAMDA
#define CGROM_mu	0xB8	// μ ce bc &mu; 	GREEK SMALL LETTER MU
#define CGROM_micro	0xB8	// µ c2 b5 &micro;	MICRO SIGN
#define CGROM_pi	0xB9	// π cf 80 &pi; 	GREEK SMALL LETTER PI
#define CGROM_rho	0xBA	// ρ cf 81 &rho;	GREEK SMALL LETTER RHO
#define CGROM_delta	0xBB	// δ ce b4 &delta;	GREEK SMALL LETTER DELTA
#define CGROM_be	0xBB	// б d0 b1		CYRILLIC SMALL LETTER BE
#define CGROM_zeta	0xBC	// ζ ce b6 &zeta;	GREEK SMALL LETTER ZETA
#define CGROM_Phi	0xBD	// Φ ce a6 &Phi;	GREEK CAPITAL LETTER PHI
//#define CGROM_Ef	0xBD	// Ф d0 a4		CYRILLIC CAPITAL LETTER EF
#define CGROM_Omega	0xBE	// Ω ce a9 &Omega;	GREEK CAPITAL LETTER OMEGA
#define CGROM_Sigma	0xBF	// Σ ce a3 &Sigma;	GREEK CAPITAL LETTER SIGMA
#define CGROM_sum	0xBF	// ∑ e2 88 91 &sum;	N-ARY SUMMATION
#define CGROM_sect	0xC0	// § c2 a7 &sect;	SECTION SIGN
#define CGROM_Ed	0xC1	// 			DOUBLE_STRUCK CAPITAL E (IE)
#define CGROM_Rd	0xC2	// ℝ e2 84 9d		DOUBLE-STRUCK CAPITAL R (IR)
#define CGROM_int	0xC3	// ∫ e2 88 ab &int;	INTEGRAL
#define CGROM_xmacr	0xC4	//			LATIN SMALL LETTER X WITH MACRON
#define CGROM_sup1	0xC5	// ¹ c2 b9 &sup1;	SUPERSCRIPT ONE (-1 with UFS)
#define CGROM_sup2	0xC6	// ² c2 b2 &sup2;	SUPERSCRIPT TWO
#define CGROM_sup3	0xC7	// ³ c2 b3 &sup3;	SUPERSCRIPT THREE
#define CGROM_supx	0xC8	// 			SUPERSCRIPT LATIN LETTER X / ASTERISK
#define CGROM_radic	0xC9	// √ e2 88 9a &radic;	SQUARE ROOT
#define CGROM_plusmn	0xCA	// ± c2 b1 &plusmn;	PLUS-MINUS SIGN
#define CGROM_uarr	0xCB	// ↑ e2 86 91 &uarr;	UPWARDS ARROW
#define CGROM_darr	0xCC	// ↓ e2 86 93 &darr;	DOWNWARDS ARROW
// CD to CF aren't assigned
#define CGROM_Agrave	0xD0	// À c3 80 &Agrave;	LATIN CAPITAL LETTER A WITH GRAVE
#define CGROM_Egrave	0xD1	// È c3 88 &Egrave;	LATIN CAPITAL LETTER E WITH GRAVE
#define CGROM_Ugrave	0xD2	// Ù c3 99 &Ugrave;	LATIN CAPITAL LETTER U WITH GRAVE
#define CGROM_Euml	0xD3	// Ë c3 8b &Euml;	LATIN CAPITAL LETTER E WITH DIAERESIS
#define CGROM_Iuml	0xD4	// Ï c3 8f &Iuml; 	LATIN CAPITAL LETTER I WITH DIAERESIS
#define CGROM_Atilde	0xD5	// Ã c3 83 &Atilde;	LATIN CAPITAL LETTER A WITH TILDE
#define CGROM_Amacr	0xD5	// Ā c4 80		LATIN CAPITAL LETTER A WITH MACRON
#define CGROM_Emacr	0xD6
#define CGROM_Imacr	0xD7
#define CGROM_Otilde	0xD8	// Õ c3 95 &Otilde;	LATIN CAPITAL LETTER O WITH TILDE
#define CGROM_Omacr	0xD8
#define CGROM_Umacr	0xD9
#define CGROM_Be	0xDA	// Б d0 91		CYRILLIC CAPITAL LETTER BE
#define CGROM_Gamma	0xDB	// Γ ce 93 &Gamma;	GREEK CAPITAL LETTER GAMMA
#define CGROM_Ghe	0xDB	// Г d0 93		CYRILLIC CAPITAL LETTER GHE
#define CGROM_De	0xDC	// Д d0 94		CYRILLIC CAPITAL LETTER DE
#define CGROM_Zhe	0xDD	// Ж d0 96		CYRILLIC CAPITAL LETTER ZHE
#define CGROM_Ze	0xDE	// З d0 97		CYRILLIC CAPITAL LETTER ZE
#define CGROM_Icyr	0xDF	// И d0 98		CYRILLIC CAPITAL LETTER I
#define CGROM_Icyrs	0xE0	// Й d0 99		CYRILLIC CAPITAL LETTER SHORT I
#define CGROM_El	0xE1	// Л d0 9b		CYRILLIC CAPITAL LETTER EL
#define CGROM_Pi	0xE2	// Π ce a0 &Pi;		GREEK CAPITAL LETTER PI
#define CGROM_Pe	0xE2	// П d0 9f		CYRILLIC CAPITAL LETTER PE
#define CGROM_prod	0xE2	// ∏ e2 88 8f &prod;	N-ARY PRODUCT
#define CGROM_Ucyr	0xE3	// У d0 a3		CYRILLIC CAPITAL LETTER U
#define CGROM_Ef	0xE4	// Ф d0 a4		CYRILLIC CAPITAL LETTER EF
#define CGROM_Tse	0xE5	// Ц d0 a6		CYRILLIC CAPITAL LETTER TSE
#define CGROM_Che	0xE6	// Ч d0 a7		CYRILLIC CAPITAL LETTER CHE
#define CGROM_Sha	0xE7	// Ш d0 a8		CYRILLIC CAPITAL LETTER SHA
#define CGROM_Shcha	0xE8	// Щ d0 a9		CYRILLIC CAPITAL LETTER SHCHA
#define CGROM_Psi	0xE8	// Ψ ce a8 &Psi;	GREEK CAPITAL LETTER PSI
#define CGROM_Hardsign	0xE9	// Ъ d0 aa		CYRILLIC CAPITAL LETTER HARD SIGN
#define CGROM_Yeru	0xEA	// Ы d0 ab		CYRILLIC CAPITAL LETTER YERU
#define CGROM_Ecyr	0xEB	// Э d0 ad		CYRILLIC CAPITAL LETTER E
#define CGROM_Yu	0xEC	// Ю d0 ae		CYRILLIC CAPITAL LETTER YU
#define CGROM_Ya	0xED	// Я d0 af		CYRILLIC CAPITAL LETTER YA
//EE not assigned
#define CGROM_deg	0xEF	// ° c2 b0 &deg;	DEGREE SIGN
#define CGROM_le	0xF0	// ≤ e2 89 a4 &le;	LESS-THAN OR EQUAL TO
#define CGROM_ge	0xF1	// ≥ e2 89 a5 &ge;	GREATER-THAN OR EQUAL TO
#define CGROM_ne	0xF2	// ≠ e2 89 a0 &ne;	NOT EQUAL TO (reversed slash on UFS))
#define CGROM_perp	0xF3	// ⊥ e2 8a a5 &perp;	UP TACK
#define CGROM_infin	0xF4	// ∞ e2 88 9e &infin;	INFINITY
#define CGROM_revtilde	0xF5	// ∽ e2 88 bd		REVERSED TILDE
#define CGROM_equiv	0xF6	// ≡ e2 89 a1 &equiv;	IDENTICAL TO
#define CGROM_Xi	0xF6	// Ξ ce 9e &Xi; 	GREEK CAPITAL LETTER XI (should be readable using this)
#define CGROM_oplus	0xF7	// ⊕ e2 8a 95 &oplus;	CIRCLED PLUS
#define CGROM_ominus	0xF8	// ⊖ e2 8a 96		CIRCLED MINUS
#define CGROM_theta	0xF8	// θ ce b8 &theta;	GREEK SMALL LETTER THETA
#define CGROM_larr	0xF9	// ← e2 86 90 &larr;	LEFTWARDS ARROW
#define CGROM_rarr	0xFA	// → e2 86 92 &rarr;	RIGHTWARDS ARROW
#define CGROM_larrb	0xFB	// ⇤ e2 87 a4		LEFTWARDS ARROW TO BAR (|<-)
#define CGROM_rarrb	0xFC	// ⇥ e2 87 a5		RIGHTWARDS ARROW TO BAR (->|)
#define CGROM_dvert	0xFD	// ‖ e2 80 96		DOUBLE VERTICAL LINE (||)
// FE and FF aren't assigned

#endif
