/*
*	vfd.c
*
*       Based originally on vfdctl.c by nebman
*/

#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>

#include "fritzmon.h"
#include "vfd.h"
#include "cgrom.h"

#define WIN1252_SUPPORT					// Support Windows-1252 extensions to ISO-8859-1
//#define LATIN_EXA_SUPPORT				// convert Latin Extended-A UTF-8 chars
#define GREEK_SUPPORT					// convert Greek UTF-8 chars
#define CYRILLIC_SUPPORT				// convert Cyrillic UTF-8 chars

#define USE_NEW_VFD					// Use new VFD driver if present

//#define USE_CGRAM

// Defines which character to display for unsupported ones.
// Use 0xFF to skip unsupported characters.
#define UNSUPPORTED_CHAR	'?'			// replace by '?'
//#define UNSUPPORTED_CHAR	' '			// replace by space
//#define UNSUPPORTED_CHAR	0xFF			// ignore

// vfd_driver_new commands:
#define	VFD_Display_Chars 		0xc0425a00
#define	VFD_DCRAM_Write 		0xc0425a00
#define VFD_CGRAM_Write			0xc0425a01	// 
#define VFD_ADRAM_Write			0xc0425a02
#define VFD_Display_DutyCycle_Write	0xc0425a03
#define VFD_Number_Of_Digit_Set		0xc0425a04	// for initialization
#define VFD_Display_Write_On_Off	0xc0425a05	// start: 0/1 = off/on, len must be 0
#define VFD_Write_Chars			0xc0425a07
#define VFD_Init			0xc0425a08
#define VFD_Get_Properties		0xc0425a09	// return -14 (not supported with UFS display type)
#define VFD_IconDisplayOnOff		0xc0425a0a	// may use ADRAM_Write instead

//#define	VFD_DCRAM_Write 	0xC0425A00
//#define	VFD_Display_Chars 	0xc0425a00
//#define VFDWRITECGRAM		0x40425A01
//#define VFD_SYMBOL???		0x40425A02
//#define VFDICONDISPLAYONOFF	0xc0425a0a
//#define VFDICONGETSTATE		0xc0425a0b

// Kernel module commands:
//#define VFDDCRAMWRITE		0xc0425a00
//#define VFDBRIGHTNESS		0xc0425a03
//#define VFDDISPLAYWRITEONOFF	0xc0425a05
//#define VFDDRIVERINIT		0xc0425a08
//#define VFDICONDISPLAYONOFF		0xc0425a0a

// VFD_Write_Chars commands (add/or address bits to these)
#define VFD_DCRAM_WRITE_CMD		0x20	// addr = 0 to 0x40
#define VFD_CGRAM_WRITE_CMD		0x40	// addr = 0 to 7
#define VFD_ADRAM_WRITE_CMD		0x60

#define VFD_DISPLAY_ON_CMD		0xE8	// ioctl.len must be 0
#define VFD_DISPLAY_OFF_CMD		0xEA	// ioctl.len must be 0

#define SLEEPTIME			200000


// vfd display chars ioctl data structure
typedef struct {
	unsigned char start;
	unsigned char data[64];
	unsigned char length;
} vfdctl_wrdisp;

#ifdef USE_NEW_VFD
static int vfd_type = 0;
#endif
static const char * vfd_name = NULL;

// Char conversion tables.

// ISO-8859-1 codes 0xA0 to 0xBF and UTF-8 codes 0xC2A0 to 0xC2BF
static unsigned char sym_table[0x20] = {
	' ',			// nbsp
	CGROM_iexcl,		// 
	CGROM_cent,		// 
	CGROM_pound,		// 
	UNSUPPORTED_CHAR,	// curren
	CGROM_yen,		// 
	'|',			// brvbar
	CGROM_sect,		// §
	UNSUPPORTED_CHAR,	// uml
	UNSUPPORTED_CHAR,	// copy
	CGROM_ordf,		// underlined on UFS display
	CGROM_laquo,		// 
	CGROM_not,		// 
	'-',			// shy
	UNSUPPORTED_CHAR,	// reg
	UNSUPPORTED_CHAR,	// macr
	CGROM_deg,		// 
	CGROM_plusmn,		// 
	CGROM_sup2,		// 
	CGROM_sup3,		// 
	UNSUPPORTED_CHAR,	// acute
	CGROM_micro,		// 
	UNSUPPORTED_CHAR,	// para
	'.',			// middot
	UNSUPPORTED_CHAR,	// cedil
	CGROM_sup1,		// -1 on UFS display
	CGROM_ordm,		// underlined on UFS display
	CGROM_raquo,		// 
	CGROM_frac14,		// 
	CGROM_frac12,		// 
	UNSUPPORTED_CHAR,	// frac34
	CGROM_iquest		// 
};
// ISO-8859-1 codes 0xC0 to 0xFF and UTF-8 codes 0xC380 to 0xC3BF
static unsigned char latin_table[0x40] = {
	CGROM_Agrave,		// Agrave
	'A',			// Aacute
	CGROM_Acirc,		// Acirc
	CGROM_Atilde,		// Atilde
	CGROM_Auml,		// Auml
	CGROM_Aring,		// Aring
	CGROM_Aelig,		// AElig
	CGROM_Ccedil,		// Ccedil
	CGROM_Egrave,		// Egrave
	CGROM_Eacute,		// Eacute
	'E',			// Ecirc
	CGROM_Euml,		// Euml
	'I',			// Igrave
	'I',			// Iacute
	'I',			// Icirc
	CGROM_Iuml,		// Iuml
	'D',			// ETH
	CGROM_Ntilde,		// Ntilde
	'O',			// Ograve
	'O',			// Oacute
	'O',			// Ocirc
	CGROM_Otilde,		// Otilde
	CGROM_Ouml,		// Ouml
	'x',			// times
	'O',			// Oslash
	CGROM_Ugrave,		// Ugrave
	'U',			// Uacute
	'U',			// Ucirc
	CGROM_Uuml,		// Uuml
	'Y',			// Yacute
	UNSUPPORTED_CHAR,	// THORN
	CGROM_szlig,		// szlig
	CGROM_agrave,		// agrave
	CGROM_aacute,		// aacute
	'a',			// acirc
	CGROM_atilde,		// atilde
	CGROM_auml,		// auml
	CGROM_aring,		// aring
	CGROM_aelig,		// aelig
	CGROM_ccedil,		// ccedil
	CGROM_egrave,		// egrave
	CGROM_eacute,		// eacute
	'e',			// ecirc
	CGROM_euml,		// euml
	CGROM_igrave,		// igrave
	CGROM_iacute,		// iacute
	'i',			// icirc
	CGROM_iuml,		// iuml
	UNSUPPORTED_CHAR,	// eth
	CGROM_ntilde,		// ntilde
	'o',			// ograve
	CGROM_oacute,		// oacute
	'o',			// ocirc
	CGROM_otilde,		// otilde
	CGROM_ouml,		// ouml
	0xF2,			// divide
	'o',			// oslash
	CGROM_ugrave,		// ugrave
	CGROM_uacute,		// uacute
	'u',			// ucirc
	CGROM_uuml,		// uuml
	'y',			// yacute
	UNSUPPORTED_CHAR,	// thorn
	CGROM_yuml		// yuml
};

#ifdef WIN1252_SUPPORT
// Windows-1252 codes 0x80 to 0x9F
static unsigned char win1252_table[0x20] = {
	UNSUPPORTED_CHAR,	// euro
	0xFF,			// not defined
	'\'',
	CGROM_fnof,		// fnof
	'"',
	'.',			// hellip ...
	UNSUPPORTED_CHAR,	// dagger
	UNSUPPORTED_CHAR,	// Dagger
	'^',			// circ
	UNSUPPORTED_CHAR,	// permil 0/00
	'S',			// Scaron
	'<',
	UNSUPPORTED_CHAR,	// Oelig
	0xFF,			// not defined
	'Z',			// Uppercase Z with caron
	0xFF,			// not defined
	0xFF,			// not defined
	'\'',			//
	'\'',			//
	'"',			//
	'"',			//
	'*',			// bull
	'-',			// ndash
	'-',			// mdash
	'~',			// tilde
	UNSUPPORTED_CHAR,	// trade
	's',			// scaron
	'>',
	UNSUPPORTED_CHAR,	// oelig
	0xFF,			// not defined
	'z',			// lowercase z with caron
	'Y'			// Yuml
};
#endif

#ifdef LATIN_EXA_SUPPORT
// Latin Extended-A UTF-8 codes 0xC480 to 0xC4BF
static unsigned char latin_exa1_table[0x40] = {
	CGROM_Amacr,		//U+0100 Ā c4 80 LATIN CAPITAL LETTER A WITH MACRON
	CGROM_amacr,		//U+0101 ā c4 81 LATIN SMALL LETTER A WITH MACRON
	'A',			//U+0102 Ă c4 82 LATIN CAPITAL LETTER A WITH BREVE
	'a',			//U+0103 ă c4 83 LATIN SMALL LETTER A WITH BREVE
	'A',			//U+0104 Ą c4 84 LATIN CAPITAL LETTER A WITH OGONEK
	'a',			//U+0105 ą c4 85 LATIN SMALL LETTER A WITH OGONEK
	'C',			//U+0106 Ć c4 86 LATIN CAPITAL LETTER C WITH ACUTE
	'c',			//U+0107 ć c4 87 LATIN SMALL LETTER C WITH ACUTE
	'C',			//U+0108 Ĉ c4 88 LATIN CAPITAL LETTER C WITH CIRCUMFLEX
	'c',			//U+0109 ĉ c4 89 LATIN SMALL LETTER C WITH CIRCUMFLEX
	'C',			//U+010A Ċ c4 8a LATIN CAPITAL LETTER C WITH DOT ABOVE
	'c',			//U+010B ċ c4 8b LATIN SMALL LETTER C WITH DOT ABOVE
	'C',			//U+010C Č c4 8c LATIN CAPITAL LETTER C WITH CARON
	'c',			//U+010D č c4 8d LATIN SMALL LETTER C WITH CARON
	'D',			//U+010E Ď c4 8e LATIN CAPITAL LETTER D WITH CARON
	'd',			//U+010F ď c4 8f LATIN SMALL LETTER D WITH CARON
	'D',			//U+0110 Đ c4 90 LATIN CAPITAL LETTER D WITH STROKE
	'd',			//U+0111 đ c4 91 LATIN SMALL LETTER D WITH STROKE
	CGROM_Emacr,		//U+0112 Ē c4 92 LATIN CAPITAL LETTER E WITH MACRON
	CGROM_emacr,		//U+0113 ē c4 93 LATIN SMALL LETTER E WITH MACRON
	'E',			//U+0114 Ĕ c4 94 LATIN CAPITAL LETTER E WITH BREVE
	'e',			//U+0115 ĕ c4 95 LATIN SMALL LETTER E WITH BREVE
	'E',			//U+0116 Ė c4 96 LATIN CAPITAL LETTER E WITH DOT ABOVE
	'e',			//U+0117 ė c4 97 LATIN SMALL LETTER E WITH DOT ABOVE
	'E',			//U+0118 Ę c4 98 LATIN CAPITAL LETTER E WITH OGONEK
	'e',			//U+0119 ę c4 99 LATIN SMALL LETTER E WITH OGONEK
	'E',			//U+011A Ě c4 9a LATIN CAPITAL LETTER E WITH CARON
	'e',			//U+011B ě c4 9b LATIN SMALL LETTER E WITH CARON
	'G',			//U+011C Ĝ c4 9c LATIN CAPITAL LETTER G WITH CIRCUMFLEX
	'g',			//U+011D ĝ c4 9d LATIN SMALL LETTER G WITH CIRCUMFLEX
	'G',			//U+011E Ğ c4 9e LATIN CAPITAL LETTER G WITH BREVE
	'g',			//U+011F ğ c4 9f LATIN SMALL LETTER G WITH BREVE
	'G',			//U+0120 Ġ c4 a0 LATIN CAPITAL LETTER G WITH DOT ABOVE
	'g',			//U+0121 ġ c4 a1 LATIN SMALL LETTER G WITH DOT ABOVE
	'G',			//U+0122 Ģ c4 a2 LATIN CAPITAL LETTER G WITH CEDILLA
	'g',			//U+0123 ģ c4 a3 LATIN SMALL LETTER G WITH CEDILLA
	'H',			//U+0124 Ĥ c4 a4 LATIN CAPITAL LETTER H WITH CIRCUMFLEX
	'h',			//U+0125 ĥ c4 a5 LATIN SMALL LETTER H WITH CIRCUMFLEX
	'H',			//U+0126 Ħ c4 a6 LATIN CAPITAL LETTER H WITH STROKE
	'h',			//U+0127 ħ c4 a7 LATIN SMALL LETTER H WITH STROKE
	'I',			//U+0128 Ĩ c4 a8 LATIN CAPITAL LETTER I WITH TILDE
	'i',			//U+0129 ĩ c4 a9 LATIN SMALL LETTER I WITH TILDE
	CGROM_Imacr,		//U+012A Ī c4 aa LATIN CAPITAL LETTER I WITH MACRON
	CGROM_imacr,		//U+012B ī c4 ab LATIN SMALL LETTER I WITH MACRON
	'I',			//U+012C Ĭ c4 ac LATIN CAPITAL LETTER I WITH BREVE
	'i',			//U+012D ĭ c4 ad LATIN SMALL LETTER I WITH BREVE
	'I',			//U+012E Į c4 ae LATIN CAPITAL LETTER I WITH OGONEK
	'i',			//U+012F į c4 af LATIN SMALL LETTER I WITH OGONEK
	'I',			//U+0130 İ c4 b0 LATIN CAPITAL LETTER I WITH DOT ABOVE
	'i',			//U+0131 ı c4 b1 LATIN SMALL LETTER DOTLESS I
	'I',			//U+0132 Ĳ c4 b2 LATIN CAPITAL LIGATURE IJ
	'i',			//U+0133 ĳ c4 b3 LATIN SMALL LIGATURE IJ
	'J',			//U+0134 Ĵ c4 b4 LATIN CAPITAL LETTER J WITH CIRCUMFLEX
	'j',			//U+0135 ĵ c4 b5 LATIN SMALL LETTER J WITH CIRCUMFLEX
	'K',			//U+0136 Ķ c4 b6 LATIN CAPITAL LETTER K WITH CEDILLA
	'k',			//U+0137 ķ c4 b7 LATIN SMALL LETTER K WITH CEDILLA
	'k',			//U+0138 ĸ c4 b8 LATIN SMALL LETTER KRA
	'L',			//U+0139 Ĺ c4 b9 LATIN CAPITAL LETTER L WITH ACUTE
	'l',			//U+013A ĺ c4 ba LATIN SMALL LETTER L WITH ACUTE
	'L',			//U+013B Ļ c4 bb LATIN CAPITAL LETTER L WITH CEDILLA
	'l',			//U+013C ļ c4 bc LATIN SMALL LETTER L WITH CEDILLA
	'L',			//U+013D Ľ c4 bd LATIN CAPITAL LETTER L WITH CARON
	'l',			//U+013E ľ c4 be LATIN SMALL LETTER L WITH CARON
	'L'			//U+013F Ŀ c4 bf LATIN CAPITAL LETTER L WITH MIDDLE DOT
};
// Latin Extended-A UTF-8 codes 0xC580 to 0xC5BF
static unsigned char latin_exa2_table[0x40] = {
	'l',			//U+0140 ŀ c5 80 LATIN SMALL LETTER L WITH MIDDLE DOT
	'L',			//U+0141 Ł c5 81 LATIN CAPITAL LETTER L WITH STROKE
	'l',			//U+0142 ł c5 82 LATIN SMALL LETTER L WITH STROKE
	'N',			//U+0143 Ń c5 83 LATIN CAPITAL LETTER N WITH ACUTE
	'n',			//U+0144 ń c5 84 LATIN SMALL LETTER N WITH ACUTE
	'N',			//U+0145 Ņ c5 85 LATIN CAPITAL LETTER N WITH CEDILLA
	'n',			//U+0146 ņ c5 86 LATIN SMALL LETTER N WITH CEDILLA
	'N',			//U+0147 Ň c5 87 LATIN CAPITAL LETTER N WITH CARON
	'n',			//U+0148 ň c5 88 LATIN SMALL LETTER N WITH CARON
	'n',			//U+0149 ŉ c5 89 LATIN SMALL LETTER N PRECEDED BY APOSTROPHE
	'N',			//U+014A Ŋ c5 8a LATIN CAPITAL LETTER ENG
	'n',			//U+014B ŋ c5 8b LATIN SMALL LETTER ENG
	CGROM_Omacr,		//U+014C Ō c5 8c LATIN CAPITAL LETTER O WITH MACRON
	CGROM_omacr,		//U+014D ō c5 8d LATIN SMALL LETTER O WITH MACRON
	'O',			//U+014E Ŏ c5 8e LATIN CAPITAL LETTER O WITH BREVE
	'o',			//U+014F ŏ c5 8f LATIN SMALL LETTER O WITH BREVE
	'O',			//U+0150 Ő c5 90 LATIN CAPITAL LETTER O WITH DOUBLE ACUTE
	'o',			//U+0151 ő c5 91 LATIN SMALL LETTER O WITH DOUBLE ACUTE
	'O',			//U+0152 Œ c5 92 &OElig; LATIN CAPITAL LIGATURE OE
	'o',			//U+0153 œ c5 93 &oelig; LATIN SMALL LIGATURE OE
	'R',			//U+0154 Ŕ c5 94 LATIN CAPITAL LETTER R WITH ACUTE
	'r',			//U+0155 ŕ c5 95 LATIN SMALL LETTER R WITH ACUTE
	'R',			//U+0156 Ŗ c5 96 LATIN CAPITAL LETTER R WITH CEDILLA
	'r',			//U+0157 ŗ c5 97 LATIN SMALL LETTER R WITH CEDILLA
	'R',			//U+0158 Ř c5 98 LATIN CAPITAL LETTER R WITH CARON
	'r',			//U+0159 ř c5 99 LATIN SMALL LETTER R WITH CARON
	'S',			//U+015A Ś c5 9a LATIN CAPITAL LETTER S WITH ACUTE
	's',			//U+015B ś c5 9b LATIN SMALL LETTER S WITH ACUTE
	'S',			//U+015C Ŝ c5 9c LATIN CAPITAL LETTER S WITH CIRCUMFLEX
	's',			//U+015D ŝ c5 9d LATIN SMALL LETTER S WITH CIRCUMFLEX
	'S',			//U+015E Ş c5 9e LATIN CAPITAL LETTER S WITH CEDILLA
	's',			//U+015F ş c5 9f LATIN SMALL LETTER S WITH CEDILLA
	'S',			//U+0160 Š c5 a0 &Scaron; LATIN CAPITAL LETTER S WITH CARON
	's',			//U+0161 š c5 a1 &scaron; LATIN SMALL LETTER S WITH CARON
	'T',			//U+0162 Ţ c5 a2 LATIN CAPITAL LETTER T WITH CEDILLA
	't',			//U+0163 ţ c5 a3 LATIN SMALL LETTER T WITH CEDILLA
	'T',			//U+0164 Ť c5 a4 LATIN CAPITAL LETTER T WITH CARON
	't',			//U+0165 ť c5 a5 LATIN SMALL LETTER T WITH CARON
	'T',			//U+0166 Ŧ c5 a6 LATIN CAPITAL LETTER T WITH STROKE
	't',			//U+0167 ŧ c5 a7 LATIN SMALL LETTER T WITH STROKE
	'U',			//U+0168 Ũ c5 a8 LATIN CAPITAL LETTER U WITH TILDE
	'u',			//U+0169 ũ c5 a9 LATIN SMALL LETTER U WITH TILDE
	CGROM_Umacr,		//U+016A Ū c5 aa LATIN CAPITAL LETTER U WITH MACRON
	CGROM_umacr,		//U+016B ū c5 ab LATIN SMALL LETTER U WITH MACRON
	'U',			//U+016C Ŭ c5 ac LATIN CAPITAL LETTER U WITH BREVE
	'u',			//U+016D ŭ c5 ad LATIN SMALL LETTER U WITH BREVE
	'U',			//U+016E Ů c5 ae LATIN CAPITAL LETTER U WITH RING ABOVE
	'u',			//U+016F ů c5 af LATIN SMALL LETTER U WITH RING ABOVE
	'U',			//U+0170 Ű c5 b0 LATIN CAPITAL LETTER U WITH DOUBLE ACUTE
	'u',			//U+0171 ű c5 b1 LATIN SMALL LETTER U WITH DOUBLE ACUTE
	'U',			//U+0172 Ų c5 b2 LATIN CAPITAL LETTER U WITH OGONEK
	'u',			//U+0173 ų c5 b3 LATIN SMALL LETTER U WITH OGONEK
	'W',			//U+0174 Ŵ c5 b4 LATIN CAPITAL LETTER W WITH CIRCUMFLEX
	'w',			//U+0175 ŵ c5 b5 LATIN SMALL LETTER W WITH CIRCUMFLEX
	'Y',			//U+0176 Ŷ c5 b6 LATIN CAPITAL LETTER Y WITH CIRCUMFLEX
	'y',			//U+0177 ŷ c5 b7 LATIN SMALL LETTER Y WITH CIRCUMFLEX
	'Y',			//U+0178 Ÿ c5 b8 &Yuml; LATIN CAPITAL LETTER Y WITH DIAERESIS
	'Z',			//U+0179 Ź c5 b9 LATIN CAPITAL LETTER Z WITH ACUTE
	'z',			//U+017A ź c5 ba LATIN SMALL LETTER Z WITH ACUTE
	'Z',			//U+017B Ż c5 bb LATIN CAPITAL LETTER Z WITH DOT ABOVE
	'z',			//U+017C ż c5 bc LATIN SMALL LETTER Z WITH DOT ABOVE
	'Z',			//U+017D Ž c5 bd LATIN CAPITAL LETTER Z WITH CARON
	'z',			//U+017E ž c5 be LATIN SMALL LETTER Z WITH CARON
	'f'			//U+017F ſ c5 bf LATIN SMALL LETTER LONG S
};
#endif

#ifdef GREEK_SUPPORT
// Greek codes 0xCE90 to 0xCEBF
static unsigned char greek1_table[0x30] = {
	0xFF,
	'A',			//U+0391 A ce 91 &Alpha; GREEK CAPITAL LETTER ALPHA
	'B',			//U+0392 B ce 92 &Beta; GREEK CAPITAL LETTER BETA
	CGROM_Gamma,		//U+0393 Γ ce 93 &Gamma; GREEK CAPITAL LETTER GAMMA
	CGROM_Delta,		//U+0394 Δ ce 94 &Delta; GREEK CAPITAL LETTER DELTA
	'E',			//U+0395 Ε ce 95 &Epsilon; GREEK CAPITAL LETTER EPSILON
	'Z',			//U+0396 Ζ ce 96 &Zeta; GREEK CAPITAL LETTER ZETA
	'H',			//U+0397 Η ce 97 &Eta; GREEK CAPITAL LETTER ETA
	CGROM_Theta,		//U+0398 Θ ce 98 &Theta; GREEK CAPITAL LETTER THETA
	'I',			//U+0399 Ι ce 99 &Iota; GREEK CAPITAL LETTER IOTA
	'K',			//U+039A Κ ce 9a &Kappa; GREEK CAPITAL LETTER KAPPA
// Capital lambda not present. Use small instead.
CGROM_lambda,			//U+039B Λ ce 9b &Lambda; GREEK CAPITAL LETTER LAMDA
	'M',			//U+039C Μ ce 9c &Mu; GREEK CAPITAL LETTER MU
	'N',			//U+039D Ν ce 9d &Nu; GREEK CAPITAL LETTER NU
	CGROM_Xi,		//U+039E Ξ ce 9e &Xi; GREEK CAPITAL LETTER XI
	'O',			//U+039F Ο ce 9f &Omicron; GREEK CAPITAL LETTER OMICRON
	CGROM_Pi,		//U+03A0 Π ce a0 &Pi; GREEK CAPITAL LETTER PI
	'P',			//U+03A1 Ρ ce a1 &Rho; GREEK CAPITAL LETTER RHO
	0xFF,			//U+03A2   ce a2 NOT DEFINED
	CGROM_Sigma,		//U+03A3 Σ ce a3 &Sigma; GREEK CAPITAL LETTER SIGMA
	'T',			//U+03A4 Τ ce a4 &Tau; GREEK CAPITAL LETTER TAU
	'Y',			//U+03A5 Υ ce a5 &Upsilon; GREEK CAPITAL LETTER UPSILON
	CGROM_Phi,		//U+03A6 Φ ce a6 &Phi; GREEK CAPITAL LETTER PHI
	'X',			//U+03A7 Χ ce a7 &Chi; GREEK CAPITAL LETTER CHI
CGROM_Psi,			//U+03A8 Ψ ce a8 &Psi; GREEK CAPITAL LETTER PSI
	CGROM_Omega,		//U+03A9 Ω ce a9 &Omega; GREEK CAPITAL LETTER OMEGA
	CGROM_Iuml,		//U+03AA Ϊ ce aa GREEK CAPITAL LETTER IOTA WITH DIALYTIKA
	'Y',			//U+03AB Ϋ ce ab GREEK CAPITAL LETTER UPSILON WITH DIALYTIKA
	CGROM_alpha,		//U+03AC ά ce ac GREEK SMALL LETTER ALPHA WITH TONOS
	CGROM_epsilon,		//U+03AD έ ce ad GREEK SMALL LETTER EPSILON WITH TONOS
	CGROM_eta,		//U+03AE ή ce ae GREEK SMALL LETTER ETA WITH TONOS
	CGROM_iacute,		//U+03AF ί ce af GREEK SMALL LETTER IOTA WITH TONOS
CGROM_uuml,			//U+03B0 ΰ ce b0 GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND TONOS
	CGROM_alpha,		//U+03B1 α ce b1 &alpha; GREEK SMALL LETTER ALPHA
	CGROM_beta,		//U+03B2 β ce b2 &beta; GREEK SMALL LETTER BETA
	CGROM_gamma,		//U+03B3 γ ce b3 &gamma; GREEK SMALL LETTER GAMMA
	CGROM_delta,		//U+03B4 δ ce b4 &delta; GREEK SMALL LETTER DELTA
	CGROM_epsilon,		//U+03B5 ε ce b5 &epsilon; GREEK SMALL LETTER EPSILON
	CGROM_zeta,		//U+03B6 ζ ce b6 &zeta; GREEK SMALL LETTER ZETA
	CGROM_eta,		//U+03B7 η ce b7 &eta; GREEK SMALL LETTER ETA
	CGROM_theta,		//U+03B8 θ ce b8 &theta; GREEK SMALL LETTER THETA
	'i',			//U+03B9 ι ce b9 &iota; GREEK SMALL LETTER IOTA
	'k',			//U+03BA κ ce ba &kappa; GREEK SMALL LETTER KAPPA
	CGROM_lambda,		//U+03BB λ ce bb &lambda; GREEK SMALL LETTER LAMDA
	CGROM_mu,		//U+03BC μ ce bc &mu; GREEK SMALL LETTER MU
	'v',			//U+03BD ν ce bd &nu; GREEK SMALL LETTER NU
'?',			//U+03BE ξ ce be &xi; GREEK SMALL LETTER XI
	'o'			//U+03BF ο ce bf &omicron; GREEK SMALL LETTER OMICRON
};
// Greek codes 0xCF80 to 0xCF89
static unsigned char greek2_table[0x10] = {
	CGROM_pi,		//U+03C0 π cf 80 &pi; GREEK SMALL LETTER PI
	CGROM_rho,		//U+03C1 ρ cf 81 &rho; GREEK SMALL LETTER RHO
	CGROM_sigmaf,		//U+03C2 ς cf 82 &sigmaf; GREEK SMALL LETTER FINAL SIGMA
	CGROM_sigma,		//U+03C3 σ cf 83 &sigma; GREEK SMALL LETTER SIGMA
	't',			//U+03C4 τ cf 84 &tau; GREEK SMALL LETTER TAU
	'u',			//U+03C5 υ cf 85 &upsilon; GREEK SMALL LETTER UPSILON
	'p',			//U+03C6 φ cf 86 &phi; GREEK SMALL LETTER PHI
	'x',			//U+03C7 χ cf 87 &chi; GREEK SMALL LETTER CHI
CGROM_Psi,			//U+03C8 ψ cf 88 &psi; GREEK SMALL LETTER PSI
	'w',			//U+03C9 ω cf 89 &omega; GREEK SMALL LETTER OMEGA
	CGROM_iuml,		//U+03CA ϊ cf 8a GREEK SMALL LETTER IOTA WITH DIALYTIKA
	CGROM_uuml,		//U+03CB ϋ cf 8b GREEK SMALL LETTER UPSILON WITH DIALYTIKA
	CGROM_oacute, 		//U+03CC ό cf 8c GREEK SMALL LETTER OMICRON WITH TONOS
	CGROM_uacute,		//U+03CD ύ cf 8d GREEK SMALL LETTER UPSILON WITH TONOS
	'w',			//U+03CE ώ cf 8e GREEK SMALL LETTER OMEGA WITH TONOS
UNSUPPORTED_CHAR		//U+03CF   cf 8f GREEK CAPITAL KAI SYMBOL
};
#endif

#ifdef CYRILLIC_SUPPORT
// Cyrillic codes 0xD080 to 0xD0BF
static unsigned char cyrillic1_table[0x40] = {
	CGROM_Egrave,		//U+0400 Ѐ d0 80 CYRILLIC CAPITAL LETTER IE WITH GRAVE
	CGROM_Euml,		//U+0401 Ё d0 81 CYRILLIC CAPITAL LETTER IO
'T',		//U+0402 Ђ d0 82 CYRILLIC CAPITAL LETTER DJE
CGROM_Ghe,		//U+0403 Ѓ d0 83 CYRILLIC CAPITAL LETTER GJE
	0xB4,		//U+0404 Є d0 84 CYRILLIC CAPITAL LETTER UKRAINIAN IE
	'S',		//U+0405 Ѕ d0 85 CYRILLIC CAPITAL LETTER DZE
	'I',		//U+0406 І d0 86 CYRILLIC CAPITAL LETTER BYELORUSSIAN-UKRAINIAN I
	CGROM_Iuml,		//U+0407 Ї d0 87 CYRILLIC CAPITAL LETTER YI
	'J',		//U+0408 Ј d0 88 CYRILLIC CAPITAL LETTER JE
'?',		//U+0409 Љ d0 89 CYRILLIC CAPITAL LETTER LJE
'H',		//U+040A Њ d0 8a CYRILLIC CAPITAL LETTER NJE
'T',		//U+040B Ћ d0 8b CYRILLIC CAPITAL LETTER TSHE
'K',		//U+040C Ќ d0 8c CYRILLIC CAPITAL LETTER KJE
'?',		//U+040D Ѝ d0 8d CYRILLIC CAPITAL LETTER I WITH GRAVE
CGROM_Ucyr,		//U+040E Ў d0 8e CYRILLIC CAPITAL LETTER SHORT U
'?',		//U+040F Џ d0 8f CYRILLIC CAPITAL LETTER DZHE
	'A',		//U+0410 А d0 90 CYRILLIC CAPITAL LETTER A
	CGROM_Be,	//U+0411 Б d0 91 CYRILLIC CAPITAL LETTER BE
	'B',		//U+0412 В d0 92 CYRILLIC CAPITAL LETTER VE
	CGROM_Ghe,	//U+0413 Г d0 93 CYRILLIC CAPITAL LETTER GHE
	CGROM_De,	//U+0414 Д d0 94 CYRILLIC CAPITAL LETTER DE
	'E',		//U+0415 Е d0 95 CYRILLIC CAPITAL LETTER IE
	CGROM_Zhe,	//U+0416 Ж d0 96 CYRILLIC CAPITAL LETTER ZHE
	CGROM_Ze,	//U+0417 З d0 97 CYRILLIC CAPITAL LETTER ZE
	CGROM_Icyr,	//U+0418 И d0 98 CYRILLIC CAPITAL LETTER I
	CGROM_Icyrs,	//U+0419 Й d0 99 CYRILLIC CAPITAL LETTER SHORT I
	'K',		//U+041A К d0 9a CYRILLIC CAPITAL LETTER KA
	CGROM_El,	//U+041B Л d0 9b CYRILLIC CAPITAL LETTER EL
	'M',		//U+041C М d0 9c CYRILLIC CAPITAL LETTER EM
	'H',		//U+041D Н d0 9d CYRILLIC CAPITAL LETTER EN
	'O',		//U+041E О d0 9e CYRILLIC CAPITAL LETTER O
	CGROM_Pe,	//U+041F П d0 9f CYRILLIC CAPITAL LETTER PE
	'P',		//U+0420 Р d0 a0 CYRILLIC CAPITAL LETTER ER
	'C',		//U+0421 С d0 a1 CYRILLIC CAPITAL LETTER ES
	'T',		//U+0422 Т d0 a2 CYRILLIC CAPITAL LETTER TE
	CGROM_Ucyr,	//U+0423 У d0 a3 CYRILLIC CAPITAL LETTER U
	CGROM_Ef,	//U+0424 Ф d0 a4 CYRILLIC CAPITAL LETTER EF
	'X',		//U+0425 Х d0 a5 CYRILLIC CAPITAL LETTER HA
	CGROM_Tse,	//U+0426 Ц d0 a6 CYRILLIC CAPITAL LETTER TSE
	CGROM_Che,	//U+0427 Ч d0 a7 CYRILLIC CAPITAL LETTER CHE
	CGROM_Sha,	//U+0428 Ш d0 a8 CYRILLIC CAPITAL LETTER SHA
	CGROM_Shcha,	//U+0429 Щ d0 a9 CYRILLIC CAPITAL LETTER SHCHA
	CGROM_Hardsign,	//U+042A Ъ d0 aa CYRILLIC CAPITAL LETTER HARD SIGN
	CGROM_Yeru,	//U+042B Ы d0 ab CYRILLIC CAPITAL LETTER YERU
	'b',		//U+042C Ь d0 ac CYRILLIC CAPITAL LETTER SOFT SIGN
	CGROM_Ecyr,	//U+042D Э d0 ad CYRILLIC CAPITAL LETTER E
	CGROM_Yu,	//U+042E Ю d0 ae CYRILLIC CAPITAL LETTER YU
	CGROM_Ya,	//U+042F Я d0 af CYRILLIC CAPITAL LETTER YA
	'a',		//U+0430 а d0 b0 CYRILLIC SMALL LETTER A
	CGROM_be,	//U+0431 б d0 b1 CYRILLIC SMALL LETTER BE
// Using capital letters in most cases
	'B',		//U+0432 в d0 b2 CYRILLIC SMALL LETTER VE
	CGROM_Ghe,	//U+0433 г d0 b3 CYRILLIC SMALL LETTER GHE
	CGROM_De,	//U+0434 д d0 b4 CYRILLIC SMALL LETTER DE
	'e',		//U+0435 е d0 b5 CYRILLIC SMALL LETTER IE
	CGROM_Zhe,	//U+0436 ж d0 b6 CYRILLIC SMALL LETTER ZHE
	CGROM_Ze,	//U+0437 з d0 b7 CYRILLIC SMALL LETTER ZE
	CGROM_Icyr,	//U+0438 и d0 b8 CYRILLIC SMALL LETTER I
	CGROM_Icyrs,	//U+0439 й d0 b9 CYRILLIC SMALL LETTER SHORT I
	'k',		//U+043A к d0 ba CYRILLIC SMALL LETTER KA
	CGROM_El,	//U+043B л d0 bb CYRILLIC SMALL LETTER EL
	'M',		//U+043C м d0 bc CYRILLIC SMALL LETTER EM
	'H',		//U+043D н d0 bd CYRILLIC SMALL LETTER EN
	'o',		//U+043E о d0 be CYRILLIC SMALL LETTER O
	CGROM_Pe	//U+043F п d0 bf CYRILLIC SMALL LETTER PE
};
// Cyrillic codes 0xD180 to 0xD1BF
static unsigned char cyrillic2_table[0x22] = {
	'p',		//U+0440 р d1 80 CYRILLIC SMALL LETTER ER
	'c',		//U+0441 с d1 81 CYRILLIC SMALL LETTER ES
	'T',		//U+0442 т d1 82 CYRILLIC SMALL LETTER TE
	CGROM_Ucyr,	//U+0443 у d1 83 CYRILLIC SMALL LETTER U
	CGROM_Ef,	//U+0444 ф d1 84 CYRILLIC SMALL LETTER EF
	'x',		//U+0445 х d1 85 CYRILLIC SMALL LETTER HA
	CGROM_Tse,	//U+0446 ц d1 86 CYRILLIC SMALL LETTER TSE
	CGROM_Che,	//U+0447 ч d1 87 CYRILLIC SMALL LETTER CHE
	CGROM_Sha,	//U+0448 ш d1 88 CYRILLIC SMALL LETTER SHA
	CGROM_Shcha,	//U+0449 щ d1 89 CYRILLIC SMALL LETTER SHCHA
	CGROM_Hardsign,	//U+044A ъ d1 8a CYRILLIC SMALL LETTER HARD SIGN
	CGROM_Yeru,	//U+044B ы d1 8b CYRILLIC SMALL LETTER YERU
	'b',		//U+044C ь d1 8c CYRILLIC SMALL LETTER SOFT SIGN
	CGROM_Ecyr,	//U+044D э d1 8d CYRILLIC SMALL LETTER E
	CGROM_Yu,	//U+044E ю d1 8e CYRILLIC SMALL LETTER YU
	CGROM_Ya,	//U+044F я d1 8f CYRILLIC SMALL LETTER YA
	CGROM_egrave,	//U+0450 ѐ d1 90 CYRILLIC SMALL LETTER IE WITH GRAVE
	CGROM_euml,	//U+0451 ё d1 91 CYRILLIC SMALL LETTER IO
'h',		//U+0452 ђ d1 92 CYRILLIC SMALL LETTER DJE
CGROM_Ghe,	//U+0453 ѓ d1 93 CYRILLIC SMALL LETTER GJE
	CGROM_epsilon,	//U+0454 є d1 94 CYRILLIC SMALL LETTER UKRAINIAN IE
	's',		//U+0455 ѕ d1 95 CYRILLIC SMALL LETTER DZE
	'i',		//U+0456 і d1 96 CYRILLIC SMALL LETTER BYELORUSSIAN-UKRAINIAN I
	CGROM_iuml,	//U+0457 ї d1 97 CYRILLIC SMALL LETTER YI
	'j',		//U+0458 ј d1 98 CYRILLIC SMALL LETTER JE
'?', //U+0459	љ	d1 99		CYRILLIC SMALL LETTER LJE
'?', //U+045A	њ	d1 9a		CYRILLIC SMALL LETTER NJE
'h',		//U+045B	ћ	d1 9b		CYRILLIC SMALL LETTER TSHE
'k',		//U+045C	ќ	d1 9c		CYRILLIC SMALL LETTER KJE
CGROM_Icyr,	//U+045D	ѝ	d1 9d		CYRILLIC SMALL LETTER I WITH GRAVE
CGROM_Ucyr, 	//U+045E	ў	d1 9e		CYRILLIC SMALL LETTER SHORT U
'?', //U+045F	џ	d1 9f		CYRILLIC SMALL LETTER DZHE
'W',		//U+0460	Ѡ	d1 a0		CYRILLIC CAPITAL LETTER OMEGA
	'w'		//U+0461	ѡ	d1 a1		CYRILLIC SMALL LETTER OMEGA
};
#endif

// General punctuation UTF-8 codes 0xE28080 to 0xE280BF
static unsigned char punctuation_table[0x40] = {
	' ',			//U+2000   e2 80 80 EN QUAD
	' ',			//U+2001   e2 80 81 EM QUAD
	' ',			//U+2002   e2 80 82 &ensp; EN SPACE
	' ',			//U+2003   e2 80 83 &emsp; EM SPACE
	' ',			//U+2004   e2 80 84 THREE-PER-EM SPACE
	' ',			//U+2005   e2 80 85 FOUR-PER-EM SPACE
	' ',			//U+2006   e2 80 86 SIX-PER-EM SPACE
	' ',			//U+2007   e2 80 87 FIGURE SPACE
	' ',			//U+2008   e2 80 88 PUNCTUATION SPACE
	' ',			//U+2009   e2 80 89 &thinsp; THIN SPACE
	' ',			//U+200A   e2 80 8a HAIR SPACE
	0xFF,			//U+200B   e2 80 8b ZERO WIDTH SPACE
	0xFF,			//U+200C‌   e2 80 8c &zwnj; ZERO WIDTH NON-JOINER
	0xFF,			//U+200D   e2 80 8d &zwj; ZERO WIDTH JOINER
	0xFF,			//U+200E   e2 80 8e &lrm; LEFT-TO-RIGHT MARK
	0xFF,			//U+200F   e2 80 8f &rlm; RIGHT-TO-LEFT MARK
	'-',			//U+2010 ‐ e2 80 90 HYPHEN
	'-',			//U+2011 ‑ e2 80 91 NON-BREAKING HYPHEN
	'-',			//U+2012 ‒ e2 80 92 FIGURE DASH
	'-',			//U+2013 – e2 80 93 &ndash; EN DASH
	'-',			//U+2014 — e2 80 94 &mdash; EM DASH
	'-',			//U+2015 ― e2 80 95 HORIZONTAL BAR
	CGROM_dvert,		//U+2016 ‖ e2 80 96 DOUBLE VERTICAL LINE
	'=',			//U+2017 ‗ e2 80 97 DOUBLE LOW LINE
	'\'',			//U+2018 ‘ e2 80 98 &lsquo; LEFT SINGLE QUOTATION MARK
	'\'',			//U+2019 ’ e2 80 99 &rsquo; RIGHT SINGLE QUOTATION MARK
	',',			//U+201A ‚ e2 80 9a &sbquo; SINGLE LOW-9 QUOTATION MARK
	'\'',			//U+201B ‛ e2 80 9b SINGLE HIGH-REVERSED-9 QUOTATION MARK
	'"',			//U+201C “ e2 80 9c &ldquo; LEFT DOUBLE QUOTATION MARK
	'"',			//U+201D ” e2 80 9d &rdquo; RIGHT DOUBLE QUOTATION MARK
	'"',			//U+201E „ e2 80 9e &bdquo; DOUBLE LOW-9 QUOTATION MARK
	'"',			//U+201F ‟ e2 80 9f DOUBLE HIGH-REVERSED-9 QUOTATION MARK
	UNSUPPORTED_CHAR,	//U+2020 † e2 80 a0 &dagger; DAGGER
	UNSUPPORTED_CHAR,	//U+2021 ‡ e2 80 a1 &Dagger; DOUBLE DAGGER
	'*',			//U+2022 • e2 80 a2 &bull; BULLET
	UNSUPPORTED_CHAR,	//U+2023 ‣ e2 80 a3 TRIANGULAR BULLET
	'.',			//U+2024 ․ e2 80 a4 ONE DOT LEADER
	'.',			//U+2025 ‥ e2 80 a5 TWO DOT LEADER
	'.',			//U+2026 … e2 80 a6 &hellip; HORIZONTAL ELLIPSIS
	'.',			//U+2027 ‧ e2 80 a7 HYPHENATION POINT
	0xFF,			//U+2028   e2 80 a8 LINE SEPARATOR
	0xFF,			//U+2029   e2 80 a9 PARAGRAPH SEPARATOR
	0xFF,			//U+202A   e2 80 aa LEFT-TO-RIGHT EMBEDDING
	0xFF,			//U+202B   e2 80 ab RIGHT-TO-LEFT EMBEDDING
	0xFF,			//U+202C   e2 80 ac POP DIRECTIONAL FORMATTING
	0xFF,			//U+202D   e2 80 ad LEFT-TO-RIGHT OVERRIDE
	0xFF,			//U+202E   e2 80 ae RIGHT-TO-LEFT OVERRIDE
	' ',			//U+202F   e2 80 af NARROW NO-BREAK SPACE
	UNSUPPORTED_CHAR,	//U+2030 ‰ e2 80 b0 &permil; PER MILLE SIGN
	UNSUPPORTED_CHAR,	//U+2031 ‱ e2 80 b1 PER TEN THOUSAND SIGN
	'\'',			//U+2032 ′ e2 80 b2 &prime; PRIME
	'"',			//U+2033 ″ e2 80 b3 &Prime; DOUBLE PRIME
	UNSUPPORTED_CHAR,	//U+2034 ‴ e2 80 b4 TRIPLE PRIME
	'\'',			//U+2035 ‵ e2 80 b5 REVERSED PRIME
	'"',			//U+2036 ‶ e2 80 b6 REVERSED DOUBLE PRIME
	UNSUPPORTED_CHAR,	//U+2037 ‷ e2 80 b7 REVERSED TRIPLE PRIME
	'^',			//U+2038 ‸ e2 80 b8 CARET
	'<',			//U+2039 ‹ e2 80 b9 &lsaquo; SINGLE LEFT-POINTING ANGLE QUOTATION MARK
	'>',			//U+203A › e2 80 ba &rsaquo; SINGLE RIGHT-POINTING ANGLE QUOTATION MARK
	'X',			//U+203B ※ e2 80 bb REFERENCE MARK
	'!',			//U+203C ‼ e2 80 bc DOUBLE EXCLAMATION MARK
	UNSUPPORTED_CHAR,	//U+203D ‽ e2 80 bd INTERROBANG
	UNSUPPORTED_CHAR,	//U+203E ‾ e2 80 be &oline; OVERLINE
	UNSUPPORTED_CHAR	//U+203F ‿ e2 80 bf UNDERTIE
};

int open_vfd()
{
	int fd_vfd = -1;
	
	if (vfd_name == NULL) {
#ifdef USE_NEW_VFD
		if (!access("/dev/vfd_new", F_OK)) {
			vfd_name = "/dev/vfd_new";
			vfd_type = 1;
		}
		else
#endif
			vfd_name = "/dev/vfd";
	}
	if (vfd_name != NULL) {
		if ((fd_vfd = open(vfd_name, O_RDWR)) == -1)
			log_msg(FM_LOG_ERR | FM_LOG_SYSERR, "open(%s)", vfd_name);
	}
	return fd_vfd;
}

#ifdef ALL_COMMANDS
int DisplayOnOff(int On)
{
	int fd;
	vfdctl_wrdisp data = { 0, "", 0 };

	if ((fd = vfd_open()) != -1) {
		data.start = On ? 1 : 0;
		if (ioctl(fd_vfd, VFD_Display_Write_On_Off, &data) == -1)
//		data.start = On ? VFD_DISPLAY_ON_CMD : VFD_DISPLAY_OFF_CMD;
//		if (ioctl(fd_vfd, VFD_Display_Chars, &data) == -1)
			log_msg(FM_LOG_ERR | FM_LOG_SYSERR, "ioctl(VFD_Display_Write_On_Off)");
		close(fd);
	}
	return fd == -1 ? -1 : 0;
}
#endif

#ifdef USE_CGRAM
/*
CGRAM: 
- 8 x (5 x 7) storage
- write: adr 0x40 to 0x47
- DCRAM codes 0x00 to 007
*/
// returns CGRAM address or -1 upon failure
unsigned char add_cgram(int fd_vfd, unsigned char c)
{
	static unsigned char nUsed = 0;
	static unsigned char cUsed[8];
	
	int i;
	vfdctl_wrdisp data;
	
#ifdef DEBUG
	log_msg(FM_LOG_DEBUG, "add_cgram(%#X)", c);
#endif
	// clear CGRAM (not really; clears only indices and buffer here)
	if (c == 0) {
		nUsed = 0;
		memset(cUsed, 0, 8);
		return 0xFF;
	}
	// check if character already present in CGRAM
	for (i = 0; i < 8; i++) {
		if (cUsed[i] == c)
			return (unsigned char)i;
	}
	// must add pattern to CGRAM
	// check if CGRAM memory completely filled
	if (nUsed >= 8)
		return 0xFF;
	// TODO: get pattern and copy to data[0] to data[4]
	memset(data.data, 0, 64);
//	data.data[0] = data.data[1] = data.data[2] = data.data[3] = data.data[4] = 0xFF;
//	data.data[0] = 5;
	if (c == 0xA3)
		memset(data.data , 0x55, 5);
	else
		memset(data.data , 0x00, 5);
	data.length = 5;
	data.start = nUsed + VFD_CGRAM_WRITE_CMD;
	if (ioctl(fd_vfd, VFD_Write_Chars, &data) == -1) {
		log_msg(FM_LOG_ERR | FM_LOG_SYSERR, "ioctl(VFDWRITECGRAM)");
		return 0xFF;
	}
	// This returns error 515
/*
	data.start = nUsed;
	if (ioctl(fd_vfd, VFD_CGRAM_Write, &data) == -1) {
		log_msg(FM_LOG_ERR | FM_LOG_SYSERR, "ioctl(VFDWRITECGRAM)");
		return 0xFF;
	}
*/
	return nUsed++;
}
#endif

/* Convert UTF-8 string to UFS 910 display format.
   Space-like characters (TAB, VTAB, FF, LF, CR) are replaced by a space.
   Control characters (codes < 0x20 except the above ones) are ignored.
*/
int convert_utf8(int nLen, const unsigned char *sMsg, unsigned char *s)
{
	unsigned i, j;
	unsigned char cIn, cOut, cSw;
    
	for (i = 0, j = 0; i < nLen; i++) {
		cIn = sMsg[i];
		cOut = UNSUPPORTED_CHAR;
		if ((cIn & 0xF0) == 0xF0) // 4 byte UTF-8 character
			i += 3;
		else if ((cIn & 0xE0) == 0xE0) {	// 3 byte UTF-8 character
			i += 2;
			if (cIn == 0xE2) {
				cSw = sMsg[i];
				if (sMsg[i-1] == 0x80)
					cOut = punctuation_table[cSw - 0x80];
				else if (sMsg[i-1] == 0x86) 
					switch (cSw) {
					case 0x90 : cOut = CGROM_larr; break;
					case 0x91 : cOut = CGROM_uarr; break;
					case 0x92 : cOut = CGROM_rarr; break;
					case 0x93 : cOut = CGROM_darr; break;
					case 0xA0 : cOut = CGROM_ne; break;
					case 0xA1 : cOut = CGROM_equiv; break;
					case 0xA4 : cOut = CGROM_le; break;
					case 0xA5 : cOut = CGROM_ge; break;
					}
				else if (sMsg[i-1] == 0x88) 
					switch (cSw) {
					case 0xE2 : cOut = CGROM_prod; break;
					case 0x91 : cOut = CGROM_sum; break;
					case 0x92 : cOut = '-'; break; // minus
					case 0x95 : cOut = '/'; break; // division slash
					case 0x9A : cOut = CGROM_radic; break;
					case 0x9E : cOut = CGROM_infin; break;
					}
				else if (sMsg[i-1] == 0x96) 
					switch (cSw) {
					case 0xb2 : cOut = CGROM_utri; break;
					case 0xb6 : cOut = CGROM_rtri; break;
					case 0xbc : cOut = CGROM_dtri; break;
					}
				else if (sMsg[i-1] == 0x97) 
					switch (cSw) {
					case 0x80 : cOut = CGROM_ltri; break;
					}
			}
		}
		else if ((cIn & 0xC0) == 0xC0) { // 2 byte UTF-8 character
			cSw = sMsg[++i];
			switch (cIn) {
			case 0xC2 :
				cOut = sym_table[cSw - 0xA0];
				break;
			case 0xC3 :
				cOut = latin_table[cSw - 0x80];
				break;
#ifdef LATIN_EXA_SUPPORT
			case 0xC4 :
				cOut = latin_exa1_table[cSw - 0x80];
				break;
			case 0xC5 :
				cOut = latin_exa2_table[cSw - 0x80];
				break;
#endif
			case 0xC6 :
				if (cSw == 0x91)
					cOut = 0x9F; // fnof
				break;
#ifdef GREEK_SUPPORT
			case 0xCE :
				cOut = greek1_table[cSw - 0x90];
				break;
			case 0xCF :
				if (cSw < 0x90)
					cOut = greek2_table[cSw - 0x80];
				break;
#endif
#ifdef CYRILLIC_SUPPORT
			case 0xD0 :
				cOut = cyrillic1_table[cSw - 0x80];
				break;
			case 0xD1 :
				if (cSw < 0xA2)
					cOut = cyrillic2_table[cSw - 0x80];
				break;
#endif
			}
		}
		else
			cOut = cIn;
		if (cOut != 0xFF)
			s[j++] = cOut;
	}
	return j;
}

// Convert ISO 8859-1 string to UFS-910 display format.
int convert_latin1(int nLen, const unsigned char *sMsg, unsigned char *s)
{
	unsigned i, j;
	unsigned char cIn, cOut;
    
	for (i = 0, j = 0; i < nLen; i++) {
		cIn = cOut = sMsg[i];
		if (cIn >= 0xC0) 
			cOut = latin_table[cIn - 0xC0]; 
		else if (cIn >= 0xA0) 
			cOut = sym_table[cIn - 0xA0]; 
		else if (cIn >= 0x80) 
#ifdef WIN1252_SUPPORT
			cOut = win1252_table[cIn - 0x80]; 
#else
			cOut = UNSUPPORTED_CHAR; 
#endif
		if (cOut != 0xFF)
			s[j++] = cOut;
	}
	return j;
}

// Write max. 16 characters from str to display.
// Fills up on the right with spaces if nLen < 16.
void vfd_write_str(int fd_vfd, unsigned nLen, const unsigned char* str)
{
	vfdctl_wrdisp data = { 0, "                ", 16 };

	if (nLen > 16)
	    nLen = 16;
	memcpy(data.data, str, nLen);
// When using VFD_Display_Chars, only codes 0x80 to 0x86 (umlauts, szlig) are supported
//  by /dev/vfd!
//	if (ioctl(fd_vfd, VFD_Display_Chars, &data) == -1)
//		log_msg(FM_LOG_ERR | FM_LOG_SYSERR, "ioctl(VFD_Display_Chars)");
	data.start = VFD_DCRAM_WRITE_CMD;	// write to DCRAM
	if (ioctl(fd_vfd, VFD_Write_Chars, &data) == -1)
		log_msg(FM_LOG_ERR | FM_LOG_SYSERR, "ioctl(VFD_Write_Chars)");
}

/* Write string to front display.
   If string is longer than 16 characters, scroll through and finally
    display first 16 characters.
   nConvert: Specifies if input string should be converted from UTF-8 or Latin1.
   nLen:     Length of string; can be -1. Must be set if string contains CGRAM address 0x00.
*/
int vfd_write(int nConvert, int nLen, const char *sMsg)
{
	int fd_vfd = -1;
	int i;
	unsigned char *sBuf;
	
	if (nLen < 0)
		nLen = (NULL != sMsg) ? strlen(sMsg) : 0;
	if (nLen && NULL != sMsg) {
		sBuf = (unsigned char *)malloc(nLen + 1);
		// convert special characters (input may be ISO or UTF-8)
		if (NULL != sBuf) {
			switch (nConvert) {
			case VFD_CONVERT_UTF8 :
				nLen = convert_utf8(nLen, sMsg, sBuf);
				break;
			case VFD_CONVERT_LATIN1 :
				nLen = convert_latin1(nLen, sMsg, sBuf);
				break;
//			case VFD_CONVERT_NONE :
			default :
				memcpy(sBuf, sMsg, nLen);
				break;
			}
			// open display device
			if ((fd_vfd = open_vfd()) != -1)
			{
				if (nLen > 16) {
					nLen -= 16;
    					for (i = 0; i <= nLen; i++) { // scroll text till end
						vfd_write_str(fd_vfd, 16, sBuf + i);
						usleep(SLEEPTIME);
					}
    					for (i = 1; i < 16; i++) { // scroll text with spaces from right
						vfd_write_str(fd_vfd, 16 - i , sBuf + i + nLen);
						usleep(SLEEPTIME);
					}
					nLen += 16;
				}
				vfd_write_str(fd_vfd, nLen, sBuf); // display first 16 chars
				close(fd_vfd);
			}
			free(sBuf);
		}
	}
	return fd_vfd == -1 ? -1 : 0;
}
